@@ -49,8 +49,8 @@ PQXX_COLD pqxx::binarystring::binarystring(field const &F)
 {
   unsigned char const *data{
     reinterpret_cast<unsigned char const *>(F.c_str())};
-  m_buf =
-    std::shared_ptr<unsigned char>{PQunescapeBytea(data, &m_size), PQfreemem};
+  m_buf = std::shared_ptr<unsigned char>{
+    PQunescapeBytea(data, &m_size), pqxx::internal::pq::pqfreemem};
   if (m_buf == nullptr)
     throw std::bad_alloc{};
 }
