@@ -24,22 +24,6 @@
 #include "pqxx/internal/header-post.hxx"
 
 
-pqxx::cursor_base::difference_type pqxx::cursor_base::all() noexcept
-{
-  // Implemented out-of-line so we don't fall afoul of Visual Studio defining
-  // min() and max() macros, which turn this expression into malformed code:
-  return std::numeric_limits<int>::max() - 1;
-}
-
-
-pqxx::cursor_base::difference_type pqxx::cursor_base::backward_all() noexcept
-{
-  // Implemented out-of-line so we don't fall afoul of Visual Studio defining
-  // min() and max() macros, which turn this expression into malformed code:
-  return std::numeric_limits<int>::min() + 1;
-}
-
-
 pqxx::cursor_base::cursor_base(
   connection &context, std::string_view Name, bool embellish_name) :
         m_name{embellish_name ? context.adorn_name(Name) : Name}
