@@ -33,6 +33,25 @@ void begin_copy(
       pqxx::internal::concat(
         "COPY "sv, table, "("sv, columns, ") FROM STDIN"sv));
 }
+
+
+/// Return the escape character for escaping the given special character.
+char escape_char(char special)
+{
+  switch (special)
+  {
+  case '\b': return 'b';
+  case '\f': return 'f';
+  case '\n': return 'n';
+  case '\r': return 'r';
+  case '\t': return 't';
+  case '\v': return 'v';
+  case '\\': return '\\';
+  }
+  PQXX_UNLIKELY throw pqxx::internal_error{pqxx::internal::concat(
+    "Stream escaping unexpectedly stopped at '",
+    static_cast<unsigned>(static_cast<unsigned char>(special)))};
+}
 } // namespace
 
 
@@ -51,7 +70,7 @@ pqxx::stream_to::~stream_to() noexcept
 
 void pqxx::stream_to::write_raw_line(std::string_view text)
 {
-  internal::gate::connection_stream_to{m_trans.conn()}.write_copy_line(text);
+  internal::gate::connection_stream_to{m_trans->conn()}.write_copy_line(text);
 }
 
 
@@ -85,7 +104,8 @@ pqxx::stream_to &pqxx::stream_to::operator<<(stream_from &tr)
 pqxx::stream_to::stream_to(
   transaction_base &tx, std::string_view path, std::string_view columns) :
         transaction_focus{tx, s_classname, path},
-        m_scanner{get_glyph_scanner(
+        m_finder{pqxx::internal::get_char_finder<
+          '\b', '\f', '\n', '\r', '\t', '\v', '\\'>(
           pqxx::internal::enc_group(tx.conn().encoding_id()))}
 {
   begin_copy(tx, path, columns);
@@ -99,72 +119,27 @@ void pqxx::stream_to::complete()
   {
     m_finished = true;
     unregister_me();
-    internal::gate::connection_stream_to{m_trans.conn()}.end_copy_write();
+    internal::gate::connection_stream_to{m_trans->conn()}.end_copy_write();
   }
 }
 
 
-/// Return escape letter for c's backslash sequence, or 0 if not needed.
-/** The API is a bit weird: you pass the width of the character, and its first
- * byte.  That's because we never need to escape a multibyte character anyway.
- */
-constexpr char escape(std::size_t width, char c)
-{
-  if (width == 1u)
-    switch (c)
-    {
-    case '\b': return 'b';
-    case '\f': return 'f';
-    case '\n': return 'n';
-    case '\r': return 'r';
-    case '\t': return 't';
-    case '\v': return 'v';
-    case '\\': return '\\';
-    }
-
-  PQXX_LIKELY
-  return '\0';
-}
-
-
 void pqxx::stream_to::escape_field_to_buffer(std::string_view data)
 {
-  if (not std::empty(data))
+  std::size_t const end{std::size(data)};
+  std::size_t here{0};
+  while (here < end)
   {
-    // Mark the beginning of a stretch that we can copy into our buffer in one
-    // go.  It feels like a waste to invoke generic multi-byte copies for every
-    // individual character in this loop, most of them actually probably only
-    // one byte long.
-    std::size_t begin_stretch{0};
-
-    std::size_t begin_char{0}, end;
-    // XXX: find_char<'\b', '\f', '\n', '\r', '\t', \v', '\\'>().
-    for (end = m_scanner(std::data(data), std::size(data), begin_char);
-         begin_char < std::size(data); begin_char = end,
-        end = m_scanner(std::data(data), std::size(data), begin_char))
+    auto const stop_char{m_finder(data, here)};
+    // Append any unremarkable we just skipped over.
+    m_buffer.append(std::data(data) + here, stop_char - here);
+    if (stop_char < end)
     {
-      // Escape sequence letter, if needed.
-      char const esc{escape(end - begin_char, data[begin_char])};
-      if (esc != '\0')
-      {
-        // This character needs escaping.  So, it ends any trivially copyable
-        // stretch that we may have been having.
-
-        // Copy the stretch we've built up into our buffer.
-        m_buffer.append(
-          std::data(data) + begin_stretch, begin_char - begin_stretch);
-
-        // Escape the current character.
-        m_buffer.push_back('\\');
-        m_buffer.push_back(esc);
-
-        // Start a new stretch, right after the current character.
-        begin_stretch = end;
-      }
+      m_buffer.push_back('\\');
+      m_buffer.push_back(escape_char(data[stop_char]));
     }
-    // Copy the final stretch.
-    m_buffer.append(
-      std::data(data) + begin_stretch, begin_char - begin_stretch);
+    here = stop_char + 1;
   }
+  // Terminate the field.
   m_buffer.push_back('\t');
 }
