@@ -13,6 +13,7 @@
 
 #include <cstring>
 #include <stdexcept>
+#include <utility>
 
 #include "pqxx/internal/header-pre.hxx"
 
@@ -233,7 +234,7 @@ public:
     register_me();
   }
 
-  ~command() { unregister_me(); }
+  ~command() noexcept { unregister_me(); }
 };
 } // namespace
 
@@ -517,7 +518,7 @@ std::string pqxx::transaction_base::description() const
 
 void pqxx::transaction_focus::register_me()
 {
-  pqxx::internal::gate::transaction_transaction_focus{m_trans}.register_focus(
+  pqxx::internal::gate::transaction_transaction_focus{*m_trans}.register_focus(
     this);
   m_registered = true;
 }
@@ -525,7 +526,7 @@ void pqxx::transaction_focus::register_me()
 
 void pqxx::transaction_focus::unregister_me() noexcept
 {
-  pqxx::internal::gate::transaction_transaction_focus{m_trans}
+  pqxx::internal::gate::transaction_transaction_focus{*m_trans}
     .unregister_focus(this);
   m_registered = false;
 }
@@ -534,6 +535,6 @@ void pqxx::transaction_focus::unregister_me() noexcept
 void pqxx::transaction_focus::reg_pending_error(
   std::string const &err) noexcept
 {
-  pqxx::internal::gate::transaction_transaction_focus{m_trans}
+  pqxx::internal::gate::transaction_transaction_focus{*m_trans}
     .register_pending_error(err);
 }
