@@ -1,6 +1,6 @@
-#include "test_helpers.hxx"
+#include <pqxx/connection>
 
-using namespace pqxx;
+#include "test_helpers.hxx"
 
 // Test program for libpqxx.  Test adorn_name.
 
@@ -8,14 +8,14 @@ namespace
 {
 void test_090()
 {
-  connection conn;
+  pqxx::connection conn;
 
   // Test connection's adorn_name() function for uniqueness
-  const std::string nametest = "basename";
+  std::string const nametest{"basename"};
 
-  PQXX_CHECK_NOT_EQUAL(conn.adorn_name(nametest),
-	conn.adorn_name(nametest),
-	"\"Unique\" names are not unique.");
+  PQXX_CHECK_NOT_EQUAL(
+    conn.adorn_name(nametest), conn.adorn_name(nametest),
+    "\"Unique\" names are not unique.");
 }
 } // namespace
 
