@@ -177,6 +177,13 @@ void test_blob_read_reads_data()
 }
 
 
+/// Cast a `char` or `std::byte` to `unsigned int`.
+template<typename BYTE> inline unsigned byte_val(BYTE val)
+{
+  return unsigned(static_cast<unsigned char>(val));
+}
+
+
 void test_blob_read_span()
 {
 #if defined(PQXX_HAVE_SPAN)
@@ -201,9 +208,9 @@ void test_blob_read_span()
   PQXX_CHECK_EQUAL(
     std::size(output), 2u, "Got unexpected buf size from blob::read().");
   PQXX_CHECK_EQUAL(
-    output[0], std::byte{'u'}, "Unexpected byte from blob::read().");
+    byte_val(output[0]), byte_val('u'), "Unexpected byte from blob::read().");
   PQXX_CHECK_EQUAL(
-    output[1], std::byte{'v'}, "Unexpected byte from blob::read().");
+    byte_val(output[1]), byte_val('v'), "Unexpected byte from blob::read().");
 
   string_buf.resize(100);
   output = b.read(std::span<std::byte>{string_buf.data(), 1});
@@ -211,7 +218,7 @@ void test_blob_read_span()
     std::size(output), 1u,
     "Did blob::read() follow string size instead of span size?");
   PQXX_CHECK_EQUAL(
-    output[0], std::byte{'w'}, "Unexpected byte from blob::read().");
+    byte_val(output[0]), byte_val('w'), "Unexpected byte from blob::read().");
 
   std::vector<std::byte> vec_buf;
   vec_buf.resize(2);
@@ -219,14 +226,14 @@ void test_blob_read_span()
   PQXX_CHECK_EQUAL(
     std::size(output2), 2u, "Got unexpected buf size from blob::read().");
   PQXX_CHECK_EQUAL(
-    output2[0], std::byte{'x'}, "Unexpected byte from blob::read().");
+    byte_val(output2[0]), byte_val('x'), "Unexpected byte from blob::read().");
   PQXX_CHECK_EQUAL(
-    output2[1], std::byte{'y'}, "Unexpected byte from blob::read().");
+    byte_val(output2[1]), byte_val('y'), "Unexpected byte from blob::read().");
 
   vec_buf.resize(100);
   output2 = b.read(vec_buf);
   PQXX_CHECK_EQUAL(std::size(output2), 1u, "Weird things happened at EOF.");
-  PQXX_CHECK_EQUAL(output2[0], std::byte{'z'}, "Bad data at EOF.");
+  PQXX_CHECK_EQUAL(byte_val(output2[0]), byte_val('z'), "Bad data at EOF.");
 #endif // PQXX_HAVE_SPAN
 }
 
@@ -246,7 +253,7 @@ void test_blob_reads_vector()
     std::size(out), std::size(content),
     "Got wrong length back when reading as vector.");
   PQXX_CHECK_EQUAL(
-    out[0], std::byte{'a'}, "Got bad data when reading as vector.");
+    byte_val(out[0]), byte_val('a'), "Got bad data when reading as vector.");
 }
 
 
@@ -303,8 +310,10 @@ void test_blob_writes_span()
   auto out{b.read(std::span<std::byte>{buf.data(), 4u})};
   PQXX_CHECK_EQUAL(
     std::size(out), 3u, "Did not get expected number of bytes back.");
-  PQXX_CHECK_EQUAL(out[0], std::byte{'f'}, "Data did not come back right.");
-  PQXX_CHECK_EQUAL(out[2], std::byte{'l'}, "Data started right, ended wrong!");
+  PQXX_CHECK_EQUAL(
+    byte_val(out[0]), byte_val('f'), "Data did not come back right.");
+  PQXX_CHECK_EQUAL(
+    byte_val(out[2]), byte_val('l'), "Data started right, ended wrong!");
 #endif // PQXX_HAVE_SPAN
 }
 
@@ -374,18 +383,20 @@ void test_blob_seek_sets_positions()
   b.seek_rel(3);
   b.read(buf, 1u);
   PQXX_CHECK_EQUAL(
-    buf[0], std::byte{3},
+    byte_val(buf[0]), byte_val(3),
     "seek_rel() from beginning did not take us to the right position.");
 
   b.seek_abs(2);
   b.read(buf, 1u);
   PQXX_CHECK_EQUAL(
-    buf[0], std::byte{2}, "seek_abs() did not take us to the right position.");
+    byte_val(buf[0]), byte_val(2),
+    "seek_abs() did not take us to the right position.");
 
   b.seek_end(-2);
   b.read(buf, 1u);
   PQXX_CHECK_EQUAL(
-    buf[0], std::byte{8}, "seek_end() did not take us to the right position.");
+    byte_val(buf[0]), byte_val(8),
+    "seek_end() did not take us to the right position.");
 }
 
 
@@ -420,7 +431,7 @@ namespace
 /** This is just here to stop Visual Studio from advertising its own
  * alternative.
  */
-std::unique_ptr<FILE, std::function<int(FILE *)>>
+std::unique_ptr<FILE, int (*)(FILE *)>
 my_fopen(char const *path, char const *mode)
 {
 #if defined(_MSC_VER)
@@ -456,7 +467,7 @@ void write_file(char const path[], std::basic_string_view<std::byte> data)
       std::fwrite(
         reinterpret_cast<char const *>(data.data()), 1, std::size(data),
         f.get()) < std::size(data))
-      std::runtime_error{"File write failed."};
+      throw std::runtime_error{"File write failed."};
   }
   catch (const std::exception &)
   {
