@@ -149,12 +149,17 @@ void test_transaction_query01()
   pqxx::connection c;
   pqxx::work w{c};
 
-  std::optional<std::tuple<int>> o;
-  o = w.query01<int>("SELECT * FROM generate_series(1, 1) AS i WHERE i = 5");
+  std::optional<std::tuple<int>> o{
+    w.query01<int>("SELECT * FROM generate_series(1, 1) AS i WHERE i = 5")};
   PQXX_CHECK(not o.has_value(), "Why did we get a row?");
   o = w.query01<int>("SELECT * FROM generate_series(8, 8)");
   PQXX_CHECK(o.has_value(), "Why did we not get a row?");
-  PQXX_CHECK_EQUAL(std::get<0>(*o), 8, "Bad value from query01().");
+
+  // The "if" is redundant (see the check above).  But gcc 11 complains when
+  // enabling maintainer mode but not audit mode: something inside the optional
+  // "may be used uninitialized."
+  if (o.has_value())
+    PQXX_CHECK_EQUAL(std::get<0>(*o), 8, "Bad value from query01().");
   PQXX_CHECK_THROWS(
     o = w.query01<int>("SELECT * FROM generate_series(1, 2)"),
     pqxx::unexpected_rows,
