@@ -3,37 +3,33 @@
 
 #include "pqxx/pqxx"
 
-using namespace pqxx;
-
 
 int main(int, char *argv[])
 {
-  lazyconnection C;
+  pqxx::connection conn;
   bool failures = false;
 
   try
   {
-    for (int i=1; argv[i]; ++i)
+    for (int i{1}; argv[i]; ++i)
     {
-      oid O;
-      from_string(argv[i], O);
+      auto o{pqxx::from_string<pqxx::oid>(argv[i])};
       try
       {
-        pqxx::perform([O, &C]{
-          work w{C};
-          largeobject l{O};
-          l.remove(w);
-          w.commit();
-          });
+        pqxx::perform([o, &conn] {
+          pqxx::work tx{conn};
+          pqxx::blob::remove(tx, o);
+          tx.commit();
+        });
       }
-      catch (const std::exception &e)
+      catch (std::exception const &e)
       {
         std::cerr << e.what() << std::endl;
         failures = true;
       }
     }
   }
-  catch (const std::exception &e)
+  catch (std::exception const &e)
   {
     std::cerr << e.what() << std::endl;
     return 2;
